#!/bin/bash

# we're getting called by cron, so just in case
PATH=/bin:/usr/bin:$PATH

COMMAND=$(basename $0)
COMMANDDIR=$(dirname $0)
COMMANDDIR=$(cd $COMMANDDIR; pwd -P)
DATADIR=/backups
DATE=$(date +%y-%m-%d-%H-%M-%S)

LOGDIR=$DATADIR/aa-dumplog
LOG=$LOGDIR/${DATE}.log
LATESTLOG=latest.log

### create a symlink in same dir
# arguments 
# actual : full path of the actual file
# link : the link name, only basename is used, will be created in same dir
function manage_link () {
    actual=$1; shift
    latest=$1; shift
    # go in the right directory
    cd $(dirname $actual)
    actualbase=$(basename $actual)
    # just in case 
    latest=$(basename $latest)
    echo "In $(pwd) : linking $actualbase to $latest"
    rm -f $latest ; ln -s $actualbase $latest
    # go back to current dir
    cd -
}
    
# usage
# clean <where> <keeptime> [... find arguments]
function clean () {
    where=$1; shift
    keeptime=$1; shift
    echo "== In $where"
    echo "Cleaning $@ - older than $keeptime days"
    find $where "$@" -atime +$keeptime | xargs rm 
}

###

case $(hostname) in
  *build*|*kruder*|*blitz*)
    DEFAULTS="subversion" ;;
  *plc*|*private*)
    DEFAULTS="planetlab4" ;;
  *)
    echo "$COMMAND: WARNING: no known defaults for hostname $(hostname)"
    DEFAULTS=""
    ;;
esac

DB_KEEPTIME=14
DB_USER=pgsqluser

SUBVERSION_KEEPTIME=7
SUBVERSION_REPO=/svn

function usage () {
    echo "This is $COMMAND"
    echo "Usage: $COMMAND [subversion]"
    echo -e "\tkeyword 'subversion' : $SUBVERSION_REPO/ is dumped"
#    echo -e "\tkeyword 'apache' : $APACHE_ROOT/ is tared"
#    echo -e "\tkeyword 'env' : $ENV_ROOT/ is tared"
#    echo -e "\totherwise performs a dump of the given database"
    echo -e "default arguments are $DEFAULTS"
    exit 1
}

function dumpdb () {
    database=$1; shift
    output=$DATADIR/$database-$DATE.sql
    latest=$database-latest.sql
    echo "Dumping $database into $output (drop-mode)"
    pg_dump --user=$DB_USER $database > $output
    manage_link $output $latest
}

function dumpsubversion () {
    output=$DATADIR/subversion-$DATE.svndump
    latest=subversion-latest.svndump
    echo "Creating subversion archive in $output"
    svnadmin dump $SUBVERSION_REPO/ > $output
    manage_link $output $latest
}

function dumpregulardir () {
    name=$1; shift
    dumpdir=$1; shift
    output=$DATADIR/$name-$DATE.tar
    latest=$name-latest.tar
    echo "Creating tar archive of $dumpdir into $output"
    tar cf $output $dumpdir
    manage_link $output $latest
}

function main () {

    # give one or more target names on the command-line
    args=""
    if [[ -z "$@" ]] ; then
	args="$DEFAULTS"
    else
	args="$@"
    fi

    [ -n "$DEBUG" ] && set -x
    
    for arg in $args; do
        case "$arg" in
	    -*)
	        usage ;;
	    subversion)
	        dumpsubversion 
		clean $DATADIR $SUBVERSION_KEEPTIME -name "$arg\*"
		;;
#	    apache)
#	        dumpregulardir $arg $APACHE_ROOT
#		clean $DATADIR $APACHE_KEEPTIME -name "$arg\*"
#		;;
#	    env)
#	        dumpregulardir $arg $ENV_ROOT
#		clean $DATADIR $ENV_KEEPTIME -name "$arg\*"
#		;;
	    *)
	        dumpdb $arg 
		clean $DATADIR $DB_KEEPTIME -name "$arg\*"
		;;
	    esac
    done

    clean $LOGDIR 14 

    manage_link $LOG $LATESTLOG
}

##############################
# create log area 
mkdir -p $LOGDIR
chmod g+w $LOGDIR

for i in "$@" ; do
  case "$i" in
    -*)
      usage ;;
  esac
done

main "$@" > $LOG 2>&1
